using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace Ants
{
    public class Ant
    {
        float Weight = (float)(2.5 * Math.Pow(10, -6));
        const float RestingMR = 4.1F;

        const float Kappa = 0.43F;
        const float MaxKappa = Kappa * 3F;
        const float MaxEnergy = 24F;
        const float ReturnHomeFactor = MaxEnergy * -1;
        const float FeedScale = 100F;
        const float DepositRate = 0.0033F;

        const float MaxAngleChange = (float)(Math.PI / 4);
        const float InitialPheromoneAmount = 1F;
        const int MaximumPathLength = 100000;

        float fEnergy;
        float fDistance;
        float fDirection;
        float fMeanDistance;
        float fMaxDistance;
        float fStandardDevDistance;

        int fCurrentPathStep = 0;
        int fWaitCount = 0;
        bool fFoundFood;
        bool fFoundNest;
        bool fAtTarget;
        PointF fPosition;
        PointF fTarget;
        Nest fHomeNest;
        Nest fStealNest;
        Colony fColony;
        Behaviour fMode;
        Role fRole;
        World fWorld;
        Path fOwnPath;
        Path fCurrentPath;
        TargetType fTargetType;
        Pen fPen = new Pen(Color.Black);
        List<Nest> fVisitedNests = new List<Nest>();
        List<Food> fVisitedFoods = new List<Food>();

        PointF AntennaLeft
        {
            get
            {
                PointF head = fHeadPosition;
                double angle = fDirection - (Math.PI / 4.0);

                return new PointF(head.X + Pheromone.SensingDistance * ((float)Math.Sin(angle)),
                    head.Y - Pheromone.SensingDistance * ((float)Math.Cos(angle)));
            }
        }

        PointF AntennaRight
        {
            get
            {
                PointF head = fPosition;
                double angle = fDirection + (Math.PI / 4.0);

                return new PointF(head.X + Pheromone.SensingDistance * ((float)Math.Sin(angle)),
                    head.Y - Pheromone.SensingDistance * ((float)Math.Cos(angle)));
            }
        }

        PointF fHeadPosition
        {
            get
            {
                return new PointF(fPosition.X + Ant.Length * ((float)Math.Sin(fDirection)),
                    fPosition.Y - Ant.Length * ((float)Math.Cos(fDirection)));
            }
        }

        enum Behaviour
        {
            Dead = 0,
            InNest = 1,
            Foraging = 2,
            Feeding = 3,
            TakingFromNest = 4,
            GoingToTarget = 5,
            Following = 6,
            Waiting = 7
        }

        enum TargetType
        {
            NoTarget = 0,
            Nest = 1,
            Food = 2,
            Home = 3
        }

        public const float Length = 4F;
        public float Direction
        {
            get
            {
                return fDirection;
            }
            set
            {
                if ((value != Double.NaN) && !Double.IsInfinity(value))
                {
                    fDirection = value;

                    while (fDirection < 0)
                        fDirection += (float)(2 * Math.PI);

                    while (fDirection > 2 * Math.PI)
                        fDirection -= (float)(2 * Math.PI);
                }
            }
        }
        public PointF Position
        {
            get { return fPosition; }
        }
        public float MeanDistance
        {
            get { return fMeanDistance; }
        }
        public float StandardDevDistance
        {
            get { return fStandardDevDistance; }
        }
        public List<int> fWaitingTimes = new List<int>();
        public enum Role
        {
            Forager = 0,
            Lazy = 1,
            Transporter = 2
        }

        //Constructor
        public Ant(Nest aNest, Role aRole, float aSpeed = 4.2F, float aStDevSpeed = 2F)
        {
            fHomeNest = aNest;
            fOwnPath = new Path(MaximumPathLength);
            fMeanDistance = aSpeed;
            fMaxDistance = 5 * aSpeed;
            fStandardDevDistance = aStDevSpeed;

            if (aRole == Role.Forager || aRole == Role.Transporter)
            {
                fMode = Behaviour.Foraging;
                fVisitedNests.Add(fHomeNest);
            }
            if (aRole == Role.Lazy || aRole == Role.Transporter)
                fMode = Behaviour.InNest;


            fPosition = new PointF(fHomeNest.Position.X, fHomeNest.Position.Y);
            fDirection = 2 * (float)Math.PI * (float)fHomeNest.Random;
            fFoundFood = false;
            fCurrentPath = fOwnPath;
            fOwnPath.AddStep(fHomeNest.Position);
            fEnergy = 0;
            fTargetType = TargetType.NoTarget;
            fRole = aRole;
            fWorld = fHomeNest.World;
            fColony = aNest.Colony;
        }

        /// <summary>
        /// Checks type and mode of ant and carries out appropriate action
        /// </summary>
        public void Step(Graphics graphics)
        {
            switch (fRole)
            {
                case Role.Forager:
                    {
                        switch (fMode)
                        {
                            case Behaviour.Waiting:
                                {
                                    //If group foraging is taking place check for leaders etc...
                                    if (this.fColony.GroupSize != 0)
                                    {
                                        //if there are no ants in the nest that have a path to food stored 
                                        //the ant forages randomly
                                        if (this.fHomeNest.Leaders.Count == 0)
                                        {
                                            LeaveHomeNest();
                                        }

                                        //if there are ants in the nest with paths stored
                                        else
                                        {
                                            //If this ant is a leader and there are enough ants then lead them out
                                            //if not stay waiting. If ant is not a leader stay waiting
                                            if ((this.fHomeNest.Leaders.Contains(this)) &&
                                                (this.fHomeNest.WaitingAnts.Count >= this.fColony.GroupSize))
                                            {
                                                LeadGroup();
                                            }
                                        }

                                        fWaitCount++;
                                    }

                                    //If there is not group foraging ant goes back out
                                    else
                                    {
                                        LeaveHomeNest();
                                    }
                                }
                                break;

                            case Behaviour.InNest:
                                {
                                    ResetAnt();
                                    //Foragers exchange food with nest or go to waiting state
                                    if (!NestExchange())
                                    {
                                        AddToWaiting();
                                    }
                                }
                                break;

                            case Behaviour.Foraging:
                                {
                                    fWaitCount = 0;
                                    //Ant will stop foraging and head home with probability dependent on energy 
                                    if ((fHomeNest.Random * ReturnHomeFactor) <= fEnergy)
                                    {
                                        ForageMove();
                                    }
                                    else
                                    {
                                        SetOffHome();
                                    }
                                }
                                break;

                            case Behaviour.GoingToTarget:
                                {
                                    MoveToTarget();
                                }
                                break;

                            case Behaviour.Feeding:
                                {
                                    fVisitedNests.Clear();
                                    fVisitedFoods.Clear();
                                    Food FoodSource = fHomeNest.World.FindFoodAt(fPosition, Food.fRadius);

                                    //Code for output
                                    if (fWorld.FoodTime == 0)
                                    {
                                        fWorld.FoodTime = fWorld.Age;
                                    }
                                    if (!fHomeNest.FoundFoods.Contains(FoodSource) && (FoodSource != null))
                                    {
                                        fHomeNest.FoundFoods.Add(FoodSource);
                                        fHomeNest.FoundFoodAmounts.Add(0);
                                    }

                                    //Ant will stay feeding if food still exists with probability dependent on energy
                                    if (!TakeFood(FoodSource))
                                    {
                                        SetOffHome();
                                    }

                                    fDistance = 0;
                                }
                                break;

                            case Behaviour.TakingFromNest: //Foragers do not take food from nests
                                {
                                    fVisitedNests.Clear();
                                    fVisitedFoods.Clear();
                                    fStealNest = fHomeNest.World.FindNestAt(fPosition, Nest.fRadius);
                                    fVisitedNests.Add(fStealNest);

                                    //Ant will stay stealing if nest still has food with probability dependent on energy
                                    //if (!StealFood())
                                    //{
                                        //TODO: Do foragers lay trails between nests??
                                        fFoundNest = true;
                                        SetOffHome();
                                    //}
                                    fDistance = 0;
                                }
                                break;

                            case Behaviour.Following:
                                {
                                    fWaitCount = 0;
                                    fWaitingTimes.Clear();
                                    FollowPath();
                                }
                                break;
                        }

                        //Ant always uses energy according to the distance it has travelled
                        fEnergy -= (RestingMR + (0.75F * (fDistance))) * Weight;
                    }
                    break;

                //INFO: Start of lazy ants
                case Role.Lazy:
                    {
                        this.fHomeNest.TakeFood(RestingMR * Weight);
                    }
                    break;


                //INFO: Start of transporter ants
                case Role.Transporter:
                    {
                        switch (fMode)
                        {
                            case Behaviour.Waiting:
                                {
                                    //INFO: Change here change when transporters leave nest
                                    //if (fHomeNest.Random < 0.05) 
                                    //{
                                        //if (fHomeNest.Amount > 5)
                                        //{
                                        //    fHomeNest.TakeFood(1);
                                        //}
                                            LeaveHomeNest();
                                    //}
                                }
                                break;

                            case Behaviour.InNest:
                                {
                                    ResetAnt();

                                    //if (!NestExchange())
                                    //{
                                        AddToWaiting();
                                    //}

                                }
                                break;

                            case Behaviour.Foraging:
                                {
                                    PheromoneMove();
                                }
                                break;

                            case Behaviour.GoingToTarget:
                                {
                                    MoveToTarget();
                                }
                                break;

                            case Behaviour.TakingFromNest:
                                {
                                    fVisitedNests.Clear();
                                    fVisitedFoods.Clear();
                                    fStealNest = fHomeNest.World.FindNestAt(fPosition, Nest.fRadius);
                                    fVisitedNests.Add(fStealNest);

                                    //Ant will stay stealing if nest still has food with probability dependent on energy
                                    //if (!StealFood())
                                    //{
                                    //TODO: Do foragers lay trails between nests??
                                    fFoundNest = true;
                                    SetOffHome();
                                    //}
                                }
                                break;
                        }

                        //Ant always uses energy according to the distance it has travelled
                        fEnergy -= (RestingMR + (0.75F * (fDistance))) * Weight;
                    }
                    break;

                default:
                    {
                        this.fHomeNest.TakeFood(RestingMR * Weight);
                    }
                    break;
            }

            if (graphics != null)
            {
                Draw(graphics);
            }
        }

        //Reset for when ant returns to nest
        void ResetAnt()
        {
            this.fVisitedNests.Clear();
            this.fVisitedFoods.Clear();
            this.fCurrentPathStep = 0;
            this.fDistance = 0;
            this.fDirection = (float)(this.fWorld.Random * Math.PI * 2);
        }

        //Adds ant to lists of waiting ants
        void AddToWaiting()
        {
            this.fVisitedNests.Add(this.fHomeNest);

            if (this.fFoundFood)
            {
                this.fHomeNest.Leaders.Add(this);
                this.fHomeNest.WaitingAnts.Add(this);
            }
            else
            {
                this.fHomeNest.WaitingAnts.Add(this);
            }

            this.fMode = Behaviour.Waiting;
        }

        //Clears parameters and sets ant to foraging
        void LeaveHomeNest()
        {
            fMode = Behaviour.Foraging;
            fTargetType = TargetType.NoTarget;

            if (this.fRole == Role.Transporter)
            {
                fDirection = (float)(this.fWorld.Random * Math.PI * 2);
            }
            else
            {
                fDirection = fDirection = (float)(this.fWorld.Random * Math.PI * 2);
            }

            fAtTarget = false;
            fCurrentPath.Clear();
            fCurrentPathStep = 0;
            fHomeNest.WaitingAnts.Remove(this);
            fFoundFood = false;
            fFoundNest = false;
            fStealNest = null;
            
        }

        //Removes ants from nest and sets to following
        void LeadGroup()
        {
            int longest = 0;
            //Remove leading ant
            fMode = Behaviour.Following;
            fHomeNest.Leaders.Remove(this);
            fHomeNest.WaitingAnts.Remove(this);
            if (this.fWaitCount > longest)
                longest = this.fWaitCount;
            fWorld.FYShuffleList(fHomeNest.WaitingAnts);

            //Change ants that are waiting to following with this ants path
            //and remove these ants from the in nest lists
            for (int i = (fColony.GroupSize - 2); i >= 0; i--)
            {
                Ant lAnt = fHomeNest.WaitingAnts[i];
                lAnt.fCurrentPath = this.fCurrentPath.Clone();
                lAnt.fMode = Behaviour.Following;
                fHomeNest.WaitingAnts.Remove(lAnt);
                fHomeNest.Leaders.Remove(lAnt);
                if (lAnt.fWaitCount > longest)
                    longest = lAnt.fWaitCount;
            }

            //Find longest waiting time and add to world waiting times
            fWorld.WaitTimes.Add(longest);
        }

        //Moves ant according to correlated random walk
        void MoveRandom()
        {
            GetDistance();
            GetRandomDirection();
            UpdatePosition();
            CheckForTargets(Food.DetectRange);
        }

        //Moves ant towrds target with randomness
        void MoveToTarget()
        {

            //Check for being at (within detection range) target
            if (World.IsPointInRadius(fPosition, fTarget, Food.DetectRange))
            {
                fDistance = (float)(Math.Sqrt((fPosition.X - fTarget.X) * (fPosition.X - fTarget.X) + (fPosition.Y - fTarget.Y) * (fPosition.Y - fTarget.Y)));
                fPosition = fTarget;
                fAtTarget = true;
            }

            else
            {
                GetDistance();

                //If energy is < 0 and ant in on way home then ant checks for food
                if ((fEnergy <= 0) && (fMode == Behaviour.GoingToTarget) && (fTargetType == TargetType.Home))
                {
                    CheckForTargets(Food.DetectRange);
                }

                if (fTargetType == TargetType.Home)
                {
                    //If no pheromones store paths home
                    if (fColony.ForageType != Colony.ForagingType.Pheromone)
                    {
                        fCurrentPath.AddStep(fPosition);
                        ++fCurrentPathStep;
                    }

                    else
                    {
                        ///INFO: Change here to not leave trails between nests
                        //If using pheomones drop them on way home ()
                        if (fFoundFood || fFoundNest)
                            fColony.AddPheromone(fPosition, InitialPheromoneAmount, Pheromone.Type.FoundFood);
                    }
                }

                GetTargetDirection(fTarget);
                UpdatePosition();
            }

            //Once at target change mode as appropriate
            if (fAtTarget)
            {
                if (fTargetType == TargetType.Food)
                {
                    fMode = Behaviour.Feeding;
                    fWorld.ArrivalTimes.Add(fWorld.Age);
                }
                if (fTargetType == TargetType.Nest)
                {
                    fMode = Behaviour.TakingFromNest;
                    fHomeNest.StealCount++;
                }
                if (fTargetType == TargetType.Home)
                {
                    fMode = Behaviour.InNest;
                    if (fFoundNest)
                    {
                        if (fWorld.RecordPaths)
                        {
                            int value = fHomeNest.ArrivalsFromNests[fStealNest];
                            value++;
                            fHomeNest.ArrivalsFromNests[fStealNest] = value;
                        }
                    }
                }
            }
        }

        //Moves ant 
        void ForageMove()
        {
            if (fColony.ForageType != Colony.ForagingType.Pheromone)
                MoveRandom();
            else
                PheromoneMove();
        }

        //Checks for pheromones and moves ant
        void PheromoneMove()
        {
            PointF checkLeft = this.AntennaLeft;
            PointF checkRight = this.AntennaRight;

            double leftWeight = 0;
            double rightWeight = 0;
            double pheromoneWeight = 1;

            double newDirection = fDirection;

            RectangleF leftRegion = new RectangleF(checkLeft.X - Pheromone.SensingDistance, checkLeft.Y - Pheromone.SensingDistance, Pheromone.SensingDistance * 2, Pheromone.SensingDistance * 2);
            RectangleF rightRegion = new RectangleF(checkRight.X - Pheromone.SensingDistance, checkRight.Y - Pheromone.SensingDistance, Pheromone.SensingDistance * 2, Pheromone.SensingDistance * 2);

            //Pheromone weighting for movement
            double leftPheromones = fColony.CheckPheromone(Pheromone.Type.FoundFood, leftRegion);
            double rightPheromones = fColony.CheckPheromone(Pheromone.Type.FoundFood, rightRegion);

            if (leftPheromones + rightPheromones > 0)
            {
                leftWeight += leftPheromones * pheromoneWeight;
                rightWeight += rightPheromones * pheromoneWeight;
            }

            if (!(leftWeight + rightWeight > 0))
            {
                fDirection += GetVonMisesAngle(Kappa);
            }

            if (leftWeight + rightWeight != 0)
            {
                double angleChange = (rightWeight - leftWeight) / (rightWeight + leftWeight);

                if (angleChange < -1 * MaxAngleChange)
                    angleChange = -1 * MaxAngleChange;
                else if (angleChange > MaxAngleChange)
                    angleChange = MaxAngleChange;

                float Randomness = (float)(Kappa + (leftWeight + rightWeight) / 5);
                Randomness = Math.Min(Randomness, MaxKappa);
                fDirection += (float)((angleChange * MaxAngleChange) + GetVonMisesAngle(Randomness));
            }

            GetDistance();
            UpdatePosition();
            CheckForTargets(Food.DetectRange);
        }

        //Moves ant to next point in stored path
        void FollowPath()
        {
            //Until reach end of stored path move to next point on path
            if (fCurrentPathStep < fCurrentPath.Length)
            {
                fCurrentPathStep++;
                fDistance = World.FindDistance(fCurrentPath[(int)(fCurrentPath.Length - fCurrentPathStep)], fPosition);
                fPosition = fCurrentPath[(int)(fCurrentPath.Length - fCurrentPathStep)];
            }

            else
            {
                fCurrentPathStep = 0;
                fCurrentPath.Clear();
                fDistance = 0;

                if (CheckForFood(Food.DetectRange))
                {
                    fMode = Behaviour.Feeding;
                }

                else
                {
                    fTarget = fHomeNest.Position;
                    fMode = Behaviour.GoingToTarget;
                    fTargetType = TargetType.Home;
                    fAtTarget = false;
                    fFoundFood = false;
                }
            }
        }

        //Check if nests and food are within dectection range
        void CheckForTargets(float aRadius)
        {
            // Check for food but not if transporter
            if (!(this.fRole == Role.Transporter))
            {
                if (CheckForFood(aRadius))
                {
                    Food FoundFood = fHomeNest.World.FindFoodAt(fPosition, aRadius);
                    if (!fVisitedFoods.Contains(FoundFood))
                    {
                        fTarget = new PointF(FoundFood.Position.X, FoundFood.Position.Y);
                        fTargetType = TargetType.Food;
                        fMode = Behaviour.GoingToTarget;
                    }
                }
            }

                if (CheckForNest(aRadius))
                {
                    Nest FoundNest = fHomeNest.World.FindNestAt(fPosition, aRadius);
                    if (!fVisitedNests.Contains(FoundNest))
                    {
                        fTarget = new PointF(FoundNest.Position.X, FoundNest.Position.Y);
                        fTargetType = TargetType.Nest;
                        fMode = Behaviour.GoingToTarget;
                    }
                }
        }

        //Updates fDistance by drawing a normally distributed number
        //If value is less than zero or greater than six calls function again
        void GetDistance()
        {
            fDistance = fHomeNest.Normal(MeanDistance, StandardDevDistance);
            if ((fDistance > fMaxDistance) || (fDistance < 0))
                GetDistance();
        }

        //Updates fDirection by drawing a number from VonMises distribution and adding to current direction
        void GetRandomDirection()
        {
            fDirection += GetVonMisesAngle(Kappa);
        }

        //Updates fDirection by drawing a number from VonMises distribution and adding to direction of target
        void GetTargetDirection(PointF target)
        {
            float TargetDirection = (float)(Math.Atan2(target.X - fPosition.X, fPosition.Y - target.Y));

            if (TargetDirection < 0)
                TargetDirection += (float)(2 * Math.PI);

            fDirection = TargetDirection + GetVonMisesAngle(Kappa);
        }

        //Chooses new turning angle
        float GetVonMisesAngle(double kappa)
        {
            /* new turning angle chosen stochastically from distribution
            * from Best & Fisher 1979. Simulation of the von Mises Distribution. 
            * Applied Statistics 28: 152-157
            * pseudo-random angle drawn from von Mises distribution
            * the mean anlge of the distribution is given by the current heading phi
            * kappa is derived from the mean vector angle as calculated from the ant 
            * paths.*/

            double newAngle, r, z, f, c, u1, u2, u3;

            u3 = fHomeNest.Random;

            double tau = 1 + Math.Sqrt(1 + 4 * kappa * kappa);
            double rho = (tau - Math.Sqrt(2 * tau)) / (2 * kappa);
            r = (1 + rho * rho) / (2 * rho);

            while (true)
            {
                u1 = fHomeNest.Random;
                u2 = fHomeNest.Random;

                z = Math.Cos(Math.PI * u1);
                f = (1 + r * z) / (r + z);
                c = kappa * (r - f);

                if ((c * (2 - c) - u2) > 0)
                    break;

                if ((Math.Log(c / u2) + 1 - c) >= 0)
                    break;
            }

            /* acos returns values only between 0 and pi! thus, i throu out the fabs */
            newAngle = (u3 - 0.5) * Math.Acos(f);

            return (float)newAngle;
        }

        //Moves ant according to fDistance and fDirection
        void UpdatePosition()
        {
            fPosition = new PointF(fPosition.X + fDistance * ((float)Math.Sin(fDirection)),
                fPosition.Y - fDistance * ((float)Math.Cos(fDirection)));
        }

        //Returns true if there is food within aRadius of fPosition
        bool CheckForFood(float aRadius)
        {
            Food lFood = fHomeNest.World.FindFoodAt(fPosition, aRadius);

            if (lFood != null && lFood.Amount > 0)
            {
                return true;
            }

            return false;
        }

        //Returns true if there is within aRadius of fPosition
        bool CheckForNest(float aRadius)
        {
            Nest lNest = fHomeNest.World.FindNestAt(fPosition, aRadius);

            if (lNest != null)
            {
                return true;
            }

            return false;
        }

        //Exchanges food with home nest and returns false if energy crosses zero
        bool NestExchange()
        {
            bool NegativeEnergy = this.fEnergy < 0;

            if (NegativeEnergy)
            {
                this.fHomeNest.TakeFood(DepositRate);
                fEnergy += DepositRate;
            }

            if (!NegativeEnergy)
            {
                fHomeNest.DepositFood(DepositRate);
                fEnergy -= DepositRate;
            }

            if ((NegativeEnergy && (fEnergy > 0)) || (!NegativeEnergy && (fEnergy < 0)))
                return false;

            return true;
        }

        //Sets state of ant for returning home
        void SetOffHome()
        {
            fTarget = fHomeNest.Position;
            fTargetType = TargetType.Home;
            fMode = Behaviour.GoingToTarget;
            fAtTarget = false;
        }

        //Takes food from FoodSource and returns false if decision to go home
        bool TakeFood(Food FoodSource)
        {
            fVisitedFoods.Add(FoodSource);
            int indexoffood = fHomeNest.FoundFoods.IndexOf(FoodSource);

            if (((fHomeNest.Random * FeedScale) > fEnergy) && CheckForFood(Food.fRadius))
            {
                if (FoodSource.Amount > DepositRate)
                {
                    FoodSource.TakeFood(DepositRate);
                    fHomeNest.FoundFoodAmounts[indexoffood] += DepositRate;
                    fEnergy += DepositRate;
                    //fFoundFood = true;
                }
                else
                {
                    FoodSource.TakeFood(FoodSource.Amount);
                    fEnergy += FoodSource.Amount;
                    fHomeNest.FoundFoodAmounts[indexoffood] += FoodSource.Amount;
                    //fFoundFood = false;
                }
                return true;
            }
            else
            {
                return false;
            }
        }

        //Takes food from none home nest and returns false if decision to go home
        bool StealFood()
        {
            //Leaves randomly and if energy in nest is negative
            if (((fHomeNest.Random * FeedScale) > fEnergy) && (fStealNest.Amount > 0))
            {
                if (fStealNest.Amount > DepositRate)
                {
                    fStealNest.TakeNestFood(DepositRate);
                    fEnergy += DepositRate;
                    fHomeNest.StealAmount += DepositRate;
                    fFoundNest = true;
                }
                else
                {
                    fStealNest.TakeNestFood(fStealNest.Amount);
                    fEnergy += fStealNest.Amount;
                    fHomeNest.StealAmount += DepositRate;
                    fFoundNest = true;
                }
                return true;
            }

            else
            {
                return false;
            }
        }

        //Draws single ant
        public void Draw(Graphics aGraphics)
        {
            Pen aPen;
            if (this.fRole == Role.Forager)
            {
                aPen = new Pen(Color.Black);
            }
            else
            {
                aPen = new Pen(Color.DarkBlue);
            }
            PointF HeadPosition = new PointF(fPosition.X + Ant.Length * ((float)Math.Sin(Direction)), fPosition.Y - Ant.Length * ((float)Math.Cos(Direction)));
            aGraphics.DrawLine(aPen, fPosition.X, fPosition.Y, HeadPosition.X, HeadPosition.Y);
        }
    }
}
